# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["OtaUpdateListParams", "Filter"]


class OtaUpdateListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter for OTA updates (deepObject style).

    Originally: filter[status], filter[sim_card_id], filter[type]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class Filter(TypedDict, total=False):
    """Consolidated filter parameter for OTA updates (deepObject style).

    Originally: filter[status], filter[sim_card_id], filter[type]
    """

    sim_card_id: str
    """The SIM card identification UUID."""

    status: Literal["in-progress", "completed", "failed"]
    """Filter by a specific status of the resource's lifecycle."""

    type: Literal["sim_card_network_preferences"]
    """Filter by type."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .connections import (
    ConnectionsResource,
    AsyncConnectionsResource,
    ConnectionsResourceWithRawResponse,
    AsyncConnectionsResourceWithRawResponse,
    ConnectionsResourceWithStreamingResponse,
    AsyncConnectionsResourceWithStreamingResponse,
)
from .integrations import (
    IntegrationsResource,
    AsyncIntegrationsResource,
    IntegrationsResourceWithRawResponse,
    AsyncIntegrationsResourceWithRawResponse,
    IntegrationsResourceWithStreamingResponse,
    AsyncIntegrationsResourceWithStreamingResponse,
)

__all__ = [
    "ConnectionsResource",
    "AsyncConnectionsResource",
    "ConnectionsResourceWithRawResponse",
    "AsyncConnectionsResourceWithRawResponse",
    "ConnectionsResourceWithStreamingResponse",
    "AsyncConnectionsResourceWithStreamingResponse",
    "IntegrationsResource",
    "AsyncIntegrationsResource",
    "IntegrationsResourceWithRawResponse",
    "AsyncIntegrationsResourceWithRawResponse",
    "IntegrationsResourceWithStreamingResponse",
    "AsyncIntegrationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types.networks import (
    DefaultGatewayCreateResponse,
    DefaultGatewayDeleteResponse,
    DefaultGatewayRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDefaultGateway:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        default_gateway = client.networks.default_gateway.create(
            network_identifier="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(DefaultGatewayCreateResponse, default_gateway, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        default_gateway = client.networks.default_gateway.create(
            network_identifier="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            wireguard_peer_id="e66c496d-4a85-423b-8b2a-8e63fac20320",
        )
        assert_matches_type(DefaultGatewayCreateResponse, default_gateway, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.networks.default_gateway.with_raw_response.create(
            network_identifier="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        default_gateway = response.parse()
        assert_matches_type(DefaultGatewayCreateResponse, default_gateway, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.networks.default_gateway.with_streaming_response.create(
            network_identifier="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            default_gateway = response.parse()
            assert_matches_type(DefaultGatewayCreateResponse, default_gateway, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `network_identifier` but received ''"):
            client.networks.default_gateway.with_raw_response.create(
                network_identifier="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        default_gateway = client.networks.default_gateway.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(DefaultGatewayRetrieveResponse, default_gateway, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.networks.default_gateway.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        default_gateway = response.parse()
        assert_matches_type(DefaultGatewayRetrieveResponse, default_gateway, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.networks.default_gateway.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            default_gateway = response.parse()
            assert_matches_type(DefaultGatewayRetrieveResponse, default_gateway, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.networks.default_gateway.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        default_gateway = client.networks.default_gateway.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(DefaultGatewayDeleteResponse, default_gateway, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.networks.default_gateway.with_raw_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        default_gateway = response.parse()
        assert_matches_type(DefaultGatewayDeleteResponse, default_gateway, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.networks.default_gateway.with_streaming_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            default_gateway = response.parse()
            assert_matches_type(DefaultGatewayDeleteResponse, default_gateway, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.networks.default_gateway.with_raw_response.delete(
                "",
            )


class TestAsyncDefaultGateway:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        default_gateway = await async_client.networks.default_gateway.create(
            network_identifier="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(DefaultGatewayCreateResponse, default_gateway, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        default_gateway = await async_client.networks.default_gateway.create(
            network_identifier="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            wireguard_peer_id="e66c496d-4a85-423b-8b2a-8e63fac20320",
        )
        assert_matches_type(DefaultGatewayCreateResponse, default_gateway, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.networks.default_gateway.with_raw_response.create(
            network_identifier="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        default_gateway = await response.parse()
        assert_matches_type(DefaultGatewayCreateResponse, default_gateway, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.networks.default_gateway.with_streaming_response.create(
            network_identifier="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            default_gateway = await response.parse()
            assert_matches_type(DefaultGatewayCreateResponse, default_gateway, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `network_identifier` but received ''"):
            await async_client.networks.default_gateway.with_raw_response.create(
                network_identifier="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        default_gateway = await async_client.networks.default_gateway.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(DefaultGatewayRetrieveResponse, default_gateway, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.networks.default_gateway.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        default_gateway = await response.parse()
        assert_matches_type(DefaultGatewayRetrieveResponse, default_gateway, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.networks.default_gateway.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            default_gateway = await response.parse()
            assert_matches_type(DefaultGatewayRetrieveResponse, default_gateway, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.networks.default_gateway.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        default_gateway = await async_client.networks.default_gateway.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(DefaultGatewayDeleteResponse, default_gateway, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.networks.default_gateway.with_raw_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        default_gateway = await response.parse()
        assert_matches_type(DefaultGatewayDeleteResponse, default_gateway, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.networks.default_gateway.with_streaming_response.delete(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            default_gateway = await response.parse()
            assert_matches_type(DefaultGatewayDeleteResponse, default_gateway, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.networks.default_gateway.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.organizations import (
    UserListResponse,
    UserRetrieveResponse,
    UserGetGroupsReportResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUsers:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        user = client.organizations.users.retrieve(
            id="id",
        )
        assert_matches_type(UserRetrieveResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_with_all_params(self, client: Telnyx) -> None:
        user = client.organizations.users.retrieve(
            id="id",
            include_groups=True,
        )
        assert_matches_type(UserRetrieveResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.organizations.users.with_raw_response.retrieve(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(UserRetrieveResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.organizations.users.with_streaming_response.retrieve(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(UserRetrieveResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.organizations.users.with_raw_response.retrieve(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        user = client.organizations.users.list()
        assert_matches_type(SyncDefaultFlatPagination[UserListResponse], user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        user = client.organizations.users.list(
            filter_email="filter[email]",
            filter_user_status="enabled",
            include_groups=True,
            page_number=1,
            page_size=1,
        )
        assert_matches_type(SyncDefaultFlatPagination[UserListResponse], user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.organizations.users.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[UserListResponse], user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.organizations.users.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[UserListResponse], user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_groups_report(self, client: Telnyx) -> None:
        user = client.organizations.users.get_groups_report()
        assert_matches_type(UserGetGroupsReportResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_groups_report_with_all_params(self, client: Telnyx) -> None:
        user = client.organizations.users.get_groups_report(
            accept="application/json",
        )
        assert_matches_type(UserGetGroupsReportResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_groups_report(self, client: Telnyx) -> None:
        response = client.organizations.users.with_raw_response.get_groups_report()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(UserGetGroupsReportResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_groups_report(self, client: Telnyx) -> None:
        with client.organizations.users.with_streaming_response.get_groups_report() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(UserGetGroupsReportResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncUsers:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        user = await async_client.organizations.users.retrieve(
            id="id",
        )
        assert_matches_type(UserRetrieveResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncTelnyx) -> None:
        user = await async_client.organizations.users.retrieve(
            id="id",
            include_groups=True,
        )
        assert_matches_type(UserRetrieveResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.organizations.users.with_raw_response.retrieve(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(UserRetrieveResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.organizations.users.with_streaming_response.retrieve(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(UserRetrieveResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.organizations.users.with_raw_response.retrieve(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        user = await async_client.organizations.users.list()
        assert_matches_type(AsyncDefaultFlatPagination[UserListResponse], user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        user = await async_client.organizations.users.list(
            filter_email="filter[email]",
            filter_user_status="enabled",
            include_groups=True,
            page_number=1,
            page_size=1,
        )
        assert_matches_type(AsyncDefaultFlatPagination[UserListResponse], user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.organizations.users.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[UserListResponse], user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.organizations.users.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[UserListResponse], user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_groups_report(self, async_client: AsyncTelnyx) -> None:
        user = await async_client.organizations.users.get_groups_report()
        assert_matches_type(UserGetGroupsReportResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_groups_report_with_all_params(self, async_client: AsyncTelnyx) -> None:
        user = await async_client.organizations.users.get_groups_report(
            accept="application/json",
        )
        assert_matches_type(UserGetGroupsReportResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_groups_report(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.organizations.users.with_raw_response.get_groups_report()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(UserGetGroupsReportResponse, user, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_groups_report(self, async_client: AsyncTelnyx) -> None:
        async with async_client.organizations.users.with_streaming_response.get_groups_report() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(UserGetGroupsReportResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["UserTagListResponse", "Data"]


class Data(BaseModel):
    number_tags: Optional[List[str]] = None
    """A list of your tags on the given resource type.

    NOTE: The casing of the tags returned will not necessarily match the casing of
    the tags when they were added to a resource. This is because the tags will have
    the casing of the first time they were used within the Telnyx system regardless
    of source.
    """

    outbound_profile_tags: Optional[List[str]] = None
    """A list of your tags on the given resource type.

    NOTE: The casing of the tags returned will not necessarily match the casing of
    the tags when they were added to a resource. This is because the tags will have
    the casing of the first time they were used within the Telnyx system regardless
    of source.
    """


class UserTagListResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import Body, Query, Headers, NotGiven, not_given
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.region_list_response import RegionListResponse

__all__ = ["RegionsResource", "AsyncRegionsResource"]


class RegionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RegionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RegionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RegionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RegionsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegionListResponse:
        """List all regions and the interfaces that region supports"""
        return self._get(
            "/regions",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RegionListResponse,
        )


class AsyncRegionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRegionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRegionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRegionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRegionsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RegionListResponse:
        """List all regions and the interfaces that region supports"""
        return await self._get(
            "/regions",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RegionListResponse,
        )


class RegionsResourceWithRawResponse:
    def __init__(self, regions: RegionsResource) -> None:
        self._regions = regions

        self.list = to_raw_response_wrapper(
            regions.list,
        )


class AsyncRegionsResourceWithRawResponse:
    def __init__(self, regions: AsyncRegionsResource) -> None:
        self._regions = regions

        self.list = async_to_raw_response_wrapper(
            regions.list,
        )


class RegionsResourceWithStreamingResponse:
    def __init__(self, regions: RegionsResource) -> None:
        self._regions = regions

        self.list = to_streamed_response_wrapper(
            regions.list,
        )


class AsyncRegionsResourceWithStreamingResponse:
    def __init__(self, regions: AsyncRegionsResource) -> None:
        self._regions = regions

        self.list = async_to_streamed_response_wrapper(
            regions.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr
from .dtmf_type import DtmfType
from .anchorsite_override import AnchorsiteOverride

__all__ = ["TexmlApplicationUpdateParams", "Inbound", "Outbound"]


class TexmlApplicationUpdateParams(TypedDict, total=False):
    friendly_name: Required[str]
    """A user-assigned name to help manage the application."""

    voice_url: Required[str]
    """URL to which Telnyx will deliver your XML Translator webhooks."""

    active: bool
    """Specifies whether the connection can be used."""

    anchorsite_override: AnchorsiteOverride
    """
    `Latency` directs Telnyx to route media through the site with the lowest
    round-trip time to the user's connection. Telnyx calculates this time using ICMP
    ping messages. This can be disabled by specifying a site to handle all media.
    """

    call_cost_in_webhooks: bool
    """Specifies if call cost webhooks should be sent for this TeXML Application."""

    dtmf_type: DtmfType
    """Sets the type of DTMF digits sent from Telnyx to this Connection.

    Note that DTMF digits sent to Telnyx will be accepted in all formats.
    """

    first_command_timeout: bool
    """
    Specifies whether calls to phone numbers associated with this connection should
    hangup after timing out.
    """

    first_command_timeout_secs: int
    """Specifies how many seconds to wait before timing out a dial command."""

    inbound: Inbound

    outbound: Outbound

    status_callback: str
    """
    URL for Telnyx to send requests to containing information about call progress
    events.
    """

    status_callback_method: Literal["get", "post"]
    """HTTP request method Telnyx should use when requesting the status_callback URL."""

    tags: SequenceNotStr[str]
    """Tags associated with the Texml Application."""

    voice_fallback_url: str
    """
    URL to which Telnyx will deliver your XML Translator webhooks if we get an error
    response from your voice_url.
    """

    voice_method: Literal["get", "post"]
    """
    HTTP request method Telnyx will use to interact with your XML Translator
    webhooks. Either 'get' or 'post'.
    """


class Inbound(TypedDict, total=False):
    channel_limit: int
    """
    When set, this will limit the total number of inbound calls to phone numbers
    associated with this connection.
    """

    shaken_stir_enabled: bool
    """
    When enabled Telnyx will include Shaken/Stir data in the Webhook for new inbound
    calls.
    """

    sip_subdomain: str
    """
    Specifies a subdomain that can be used to receive Inbound calls to a Connection,
    in the same way a phone number is used, from a SIP endpoint. Example: the
    subdomain "example.sip.telnyx.com" can be called from any SIP endpoint by using
    the SIP URI "sip:@example.sip.telnyx.com" where the user part can be any
    alphanumeric value. Please note TLS encrypted calls are not allowed for
    subdomain calls.
    """

    sip_subdomain_receive_settings: Literal["only_my_connections", "from_anyone"]
    """
    This option can be enabled to receive calls from: "Anyone" (any SIP endpoint in
    the public Internet) or "Only my connections" (any connection assigned to the
    same Telnyx user).
    """


class Outbound(TypedDict, total=False):
    channel_limit: int
    """
    When set, this will limit the total number of outbound calls to phone numbers
    associated with this connection.
    """

    outbound_voice_profile_id: str
    """Identifies the associated outbound voice profile."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional

from ..._models import BaseModel
from ..shared.simple_sim_card import SimpleSimCard

__all__ = ["RegisterCreateResponse", "Error", "ErrorSource"]


class ErrorSource(BaseModel):
    parameter: Optional[str] = None
    """Indicates which query parameter caused the error."""

    pointer: Optional[str] = None
    """JSON pointer (RFC6901) to the offending entity."""


class Error(BaseModel):
    code: str

    title: str

    detail: Optional[str] = None

    meta: Optional[Dict[str, object]] = None

    source: Optional[ErrorSource] = None


class RegisterCreateResponse(BaseModel):
    data: Optional[List[SimpleSimCard]] = None
    """Successfully registered SIM cards."""

    errors: Optional[List[Error]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["UserAddressCreateParams"]


class UserAddressCreateParams(TypedDict, total=False):
    business_name: Required[str]
    """The business name associated with the user address."""

    country_code: Required[str]
    """The two-character (ISO 3166-1 alpha-2) country code of the user address."""

    first_name: Required[str]
    """The first name associated with the user address."""

    last_name: Required[str]
    """The last name associated with the user address."""

    locality: Required[str]
    """The locality of the user address.

    For US addresses, this corresponds to the city of the address.
    """

    street_address: Required[str]
    """The primary street address information about the user address."""

    administrative_area: str
    """The locality of the user address.

    For US addresses, this corresponds to the state of the address.
    """

    borough: str
    """The borough of the user address.

    This field is not used for addresses in the US but is used for some
    international addresses.
    """

    customer_reference: str
    """A customer reference string for customer look ups."""

    extended_address: str
    """
    Additional street address information about the user address such as, but not
    limited to, unit number or apartment number.
    """

    neighborhood: str
    """The neighborhood of the user address.

    This field is not used for addresses in the US but is used for some
    international addresses.
    """

    phone_number: str
    """The phone number associated with the user address."""

    postal_code: str
    """The postal code of the user address."""

    skip_address_verification: bool
    """
    An optional boolean value specifying if verification of the address should be
    skipped or not. UserAddresses are generally used for shipping addresses, and
    failure to validate your shipping address will likely result in a failure to
    deliver SIM cards or other items ordered from Telnyx. Do not use this parameter
    unless you are sure that the address is correct even though it cannot be
    validated. If this is set to any value other than true, verification of the
    address will be attempted, and the user address will not be allowed if
    verification fails. If verification fails but suggested values are available
    that might make the address correct, they will be present in the response as
    well. If this value is set to true, then the verification will not be attempted.
    Defaults to false (verification will be performed).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["UpdateRegulatoryRequirementParam"]


class UpdateRegulatoryRequirementParam(TypedDict, total=False):
    field_value: str
    """The value of the requirement.

    For address and document requirements, this should be the ID of the resource.
    For textual, this should be the value of the requirement.
    """

    requirement_id: str
    """Unique id for a requirement."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["ActionValidateRegistrationCodesResponse", "Data"]


class Data(BaseModel):
    invalid_detail: Optional[str] = None
    """The validation message"""

    record_type: Optional[str] = None

    registration_code: Optional[str] = None
    """The 10-digit SIM card registration code"""

    valid: Optional[bool] = None
    """The attribute that denotes whether the code is valid or not"""


class ActionValidateRegistrationCodesResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .media_resource import MediaResource
from .pagination_meta import PaginationMeta

__all__ = ["MediaListResponse"]


class MediaListResponse(BaseModel):
    data: Optional[List[MediaResource]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["PortingOrderRetrieveLoaTemplateParams"]


class PortingOrderRetrieveLoaTemplateParams(TypedDict, total=False):
    loa_configuration_id: str
    """The identifier of the LOA configuration to use for the template.

    If not provided, the default LOA configuration will be used.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["GlobalIPUsageRetrieveResponse", "Data", "DataGlobalIP", "DataReceived", "DataTransmitted"]


class DataGlobalIP(BaseModel):
    id: Optional[str] = None
    """Global IP ID."""

    ip_address: Optional[str] = None
    """The Global IP address."""


class DataReceived(BaseModel):
    amount: Optional[float] = None
    """The amount of data received."""

    unit: Optional[str] = None
    """The unit of the amount of data received."""


class DataTransmitted(BaseModel):
    amount: Optional[float] = None
    """The amount of data transmitted."""

    unit: Optional[str] = None
    """The unit of the amount of data transmitted."""


class Data(BaseModel):
    global_ip: Optional[DataGlobalIP] = None

    received: Optional[DataReceived] = None

    timestamp: Optional[datetime] = None
    """The timestamp of the metric."""

    transmitted: Optional[DataTransmitted] = None


class GlobalIPUsageRetrieveResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["SpeechToTextCreateParams"]


class SpeechToTextCreateParams(TypedDict, total=False):
    end_date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """End date in ISO format with timezone (date range must be up to one month)"""

    start_date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """Start date in ISO format with timezone"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

from ..stream_codec import StreamCodec
from ..dialogflow_config_param import DialogflowConfigParam
from ..stream_bidirectional_mode import StreamBidirectionalMode
from ..stream_bidirectional_codec import StreamBidirectionalCodec
from ..stream_bidirectional_target_legs import StreamBidirectionalTargetLegs
from ..stream_bidirectional_sampling_rate import StreamBidirectionalSamplingRate

__all__ = ["ActionStartStreamingParams"]


class ActionStartStreamingParams(TypedDict, total=False):
    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    dialogflow_config: DialogflowConfigParam

    enable_dialogflow: bool
    """Enables Dialogflow for the current call. The default value is false."""

    stream_bidirectional_codec: StreamBidirectionalCodec
    """Indicates codec for bidirectional streaming RTP payloads.

    Used only with stream_bidirectional_mode=rtp. Case sensitive.
    """

    stream_bidirectional_mode: StreamBidirectionalMode
    """Configures method of bidirectional streaming (mp3, rtp)."""

    stream_bidirectional_sampling_rate: StreamBidirectionalSamplingRate
    """Audio sampling rate."""

    stream_bidirectional_target_legs: StreamBidirectionalTargetLegs
    """Specifies which call legs should receive the bidirectional stream audio."""

    stream_codec: StreamCodec
    """Specifies the codec to be used for the streamed audio.

    When set to 'default' or when transcoding is not possible, the codec from the
    call will be used.
    """

    stream_track: Literal["inbound_track", "outbound_track", "both_tracks"]
    """Specifies which track should be streamed."""

    stream_url: str
    """The destination WebSocket address where the stream is going to be delivered."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ActionSubmitVerificationCodeParams"]


class ActionSubmitVerificationCodeParams(TypedDict, total=False):
    verification_code: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..filter import Filter
from ....._models import BaseModel

__all__ = ["CdrDetailedReqResponse"]


class CdrDetailedReqResponse(BaseModel):
    """Response object for CDR detailed report"""

    id: Optional[str] = None
    """Unique identifier for the report"""

    call_types: Optional[List[int]] = None
    """List of call types (Inbound = 1, Outbound = 2)"""

    connections: Optional[List[int]] = None
    """List of connections"""

    created_at: Optional[str] = None
    """Creation date of the report"""

    end_time: Optional[str] = None
    """End time in ISO format"""

    filters: Optional[List[Filter]] = None
    """List of filters"""

    managed_accounts: Optional[List[str]] = None
    """List of managed accounts"""

    record_type: Optional[str] = None

    record_types: Optional[List[int]] = None
    """List of record types (Complete = 1, Incomplete = 2, Errors = 3)"""

    report_name: Optional[str] = None
    """Name of the report"""

    report_url: Optional[str] = None
    """URL to download the report"""

    retry: Optional[int] = None
    """Number of retries"""

    source: Optional[str] = None
    """Source of the report.

    Valid values: calls (default), call-control, fax-api, webrtc
    """

    start_time: Optional[str] = None
    """Start time in ISO format"""

    status: Optional[int] = None
    """Status of the report (Pending = 1, Complete = 2, Failed = 3, Expired = 4)"""

    timezone: Optional[str] = None
    """Timezone for the report"""

    updated_at: Optional[str] = None
    """Last update date of the report"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["JobUpdateEmergencySettingsBatchParams"]


class JobUpdateEmergencySettingsBatchParams(TypedDict, total=False):
    emergency_enabled: Required[bool]
    """Indicates whether to enable or disable emergency services on the numbers."""

    phone_numbers: Required[SequenceNotStr[str]]

    emergency_address_id: Optional[str]
    """Identifies the address to be used with emergency services.

    Required if emergency_enabled is true, must be null or omitted if
    emergency_enabled is false.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.porting_orders import (
    AdditionalDocumentListResponse,
    AdditionalDocumentCreateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAdditionalDocuments:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        additional_document = client.porting_orders.additional_documents.create(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(AdditionalDocumentCreateResponse, additional_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        additional_document = client.porting_orders.additional_documents.create(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            additional_documents=[
                {
                    "document_id": "22771a52-c43a-4539-80db-9dd9ec36e237",
                    "document_type": "loa",
                },
                {
                    "document_id": "d91474e6-4ebc-4ec1-b379-c596eeb405d6",
                    "document_type": "invoice",
                },
            ],
        )
        assert_matches_type(AdditionalDocumentCreateResponse, additional_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.porting_orders.additional_documents.with_raw_response.create(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        additional_document = response.parse()
        assert_matches_type(AdditionalDocumentCreateResponse, additional_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.porting_orders.additional_documents.with_streaming_response.create(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            additional_document = response.parse()
            assert_matches_type(AdditionalDocumentCreateResponse, additional_document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.additional_documents.with_raw_response.create(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        additional_document = client.porting_orders.additional_documents.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(
            SyncDefaultFlatPagination[AdditionalDocumentListResponse], additional_document, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        additional_document = client.porting_orders.additional_documents.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            filter={"document_type": ["loa"]},
            page_number=0,
            page_size=0,
            sort={"value": "created_at"},
        )
        assert_matches_type(
            SyncDefaultFlatPagination[AdditionalDocumentListResponse], additional_document, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.porting_orders.additional_documents.with_raw_response.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        additional_document = response.parse()
        assert_matches_type(
            SyncDefaultFlatPagination[AdditionalDocumentListResponse], additional_document, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.porting_orders.additional_documents.with_streaming_response.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            additional_document = response.parse()
            assert_matches_type(
                SyncDefaultFlatPagination[AdditionalDocumentListResponse], additional_document, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.additional_documents.with_raw_response.list(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        additional_document = client.porting_orders.additional_documents.delete(
            additional_document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert additional_document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.porting_orders.additional_documents.with_raw_response.delete(
            additional_document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        additional_document = response.parse()
        assert additional_document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.porting_orders.additional_documents.with_streaming_response.delete(
            additional_document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            additional_document = response.parse()
            assert additional_document is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.additional_documents.with_raw_response.delete(
                additional_document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `additional_document_id` but received ''"
        ):
            client.porting_orders.additional_documents.with_raw_response.delete(
                additional_document_id="",
                id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )


class TestAsyncAdditionalDocuments:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        additional_document = await async_client.porting_orders.additional_documents.create(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(AdditionalDocumentCreateResponse, additional_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        additional_document = await async_client.porting_orders.additional_documents.create(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            additional_documents=[
                {
                    "document_id": "22771a52-c43a-4539-80db-9dd9ec36e237",
                    "document_type": "loa",
                },
                {
                    "document_id": "d91474e6-4ebc-4ec1-b379-c596eeb405d6",
                    "document_type": "invoice",
                },
            ],
        )
        assert_matches_type(AdditionalDocumentCreateResponse, additional_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.additional_documents.with_raw_response.create(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        additional_document = await response.parse()
        assert_matches_type(AdditionalDocumentCreateResponse, additional_document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.additional_documents.with_streaming_response.create(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            additional_document = await response.parse()
            assert_matches_type(AdditionalDocumentCreateResponse, additional_document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.additional_documents.with_raw_response.create(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        additional_document = await async_client.porting_orders.additional_documents.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(
            AsyncDefaultFlatPagination[AdditionalDocumentListResponse], additional_document, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        additional_document = await async_client.porting_orders.additional_documents.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            filter={"document_type": ["loa"]},
            page_number=0,
            page_size=0,
            sort={"value": "created_at"},
        )
        assert_matches_type(
            AsyncDefaultFlatPagination[AdditionalDocumentListResponse], additional_document, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.additional_documents.with_raw_response.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        additional_document = await response.parse()
        assert_matches_type(
            AsyncDefaultFlatPagination[AdditionalDocumentListResponse], additional_document, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.additional_documents.with_streaming_response.list(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            additional_document = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[AdditionalDocumentListResponse], additional_document, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.additional_documents.with_raw_response.list(
                id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        additional_document = await async_client.porting_orders.additional_documents.delete(
            additional_document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert additional_document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.additional_documents.with_raw_response.delete(
            additional_document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        additional_document = await response.parse()
        assert additional_document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.additional_documents.with_streaming_response.delete(
            additional_document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            additional_document = await response.parse()
            assert additional_document is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.additional_documents.with_raw_response.delete(
                additional_document_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `additional_document_id` but received ''"
        ):
            await async_client.porting_orders.additional_documents.with_raw_response.delete(
                additional_document_id="",
                id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_machine_premium_greeting_ended import CallMachinePremiumGreetingEnded

__all__ = ["CallMachinePremiumGreetingEndedWebhookEvent"]


class CallMachinePremiumGreetingEndedWebhookEvent(BaseModel):
    data: Optional[CallMachinePremiumGreetingEnded] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ....._models import BaseModel
from .batch_csv_pagination_meta import BatchCsvPaginationMeta
from .mdr_detail_report_response import MdrDetailReportResponse

__all__ = ["MessagingListResponse"]


class MessagingListResponse(BaseModel):
    data: Optional[List[MdrDetailReportResponse]] = None

    meta: Optional[BatchCsvPaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from ....._models import BaseModel

__all__ = ["TelnyxAgentLinkResponse", "Data"]


class Data(BaseModel):
    created_at: datetime

    run_id: str

    telnyx_agent_id: str


class TelnyxAgentLinkResponse(BaseModel):
    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["PhoneNumberUpdateParams"]


class PhoneNumberUpdateParams(TypedDict, total=False):
    billing_group_id: str
    """Identifies the billing group associated with the phone number."""

    connection_id: str
    """Identifies the connection associated with the phone number."""

    customer_reference: str
    """A customer reference string for customer look ups."""

    external_pin: str
    """
    If someone attempts to port your phone number away from Telnyx and your phone
    number has an external PIN set, we will attempt to verify that you provided the
    correct external PIN to the winning carrier. Note that not all carriers
    cooperate with this security mechanism.
    """

    hd_voice_enabled: bool
    """Indicates whether HD voice is enabled for this number."""

    tags: SequenceNotStr[str]
    """A list of user-assigned tags to help organize phone numbers."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .organizations import (
    OrganizationsResource,
    AsyncOrganizationsResource,
    OrganizationsResourceWithRawResponse,
    AsyncOrganizationsResourceWithRawResponse,
    OrganizationsResourceWithStreamingResponse,
    AsyncOrganizationsResourceWithStreamingResponse,
)

__all__ = [
    "UsersResource",
    "AsyncUsersResource",
    "UsersResourceWithRawResponse",
    "AsyncUsersResourceWithRawResponse",
    "UsersResourceWithStreamingResponse",
    "AsyncUsersResourceWithStreamingResponse",
    "OrganizationsResource",
    "AsyncOrganizationsResource",
    "OrganizationsResourceWithRawResponse",
    "AsyncOrganizationsResourceWithRawResponse",
    "OrganizationsResourceWithStreamingResponse",
    "AsyncOrganizationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ReservedPhoneNumber"]


class ReservedPhoneNumber(BaseModel):
    id: Optional[str] = None

    created_at: Optional[datetime] = None
    """
    An ISO 8901 datetime string denoting when the individual number reservation was
    created.
    """

    expired_at: Optional[datetime] = None
    """
    An ISO 8901 datetime string for when the individual number reservation is going
    to expire
    """

    phone_number: Optional[str] = None

    record_type: Optional[str] = None

    status: Optional[Literal["pending", "success", "failure"]] = None
    """The status of the phone number's reservation."""

    updated_at: Optional[datetime] = None
    """
    An ISO 8901 datetime string for when the the individual number reservation was
    updated.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal

import httpx

from ..types import requirement_group_list_params, requirement_group_create_params, requirement_group_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.requirement_group import RequirementGroup
from ..types.requirement_group_list_response import RequirementGroupListResponse

__all__ = ["RequirementGroupsResource", "AsyncRequirementGroupsResource"]


class RequirementGroupsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RequirementGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RequirementGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RequirementGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RequirementGroupsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        action: Literal["ordering", "porting"],
        country_code: str,
        phone_number_type: Literal["local", "toll_free", "mobile", "national", "shared_cost"],
        customer_reference: str | Omit = omit,
        regulatory_requirements: Iterable[requirement_group_create_params.RegulatoryRequirement] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementGroup:
        """
        Create a new requirement group

        Args:
          country_code: ISO alpha 2 country code

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/requirement_groups",
            body=maybe_transform(
                {
                    "action": action,
                    "country_code": country_code,
                    "phone_number_type": phone_number_type,
                    "customer_reference": customer_reference,
                    "regulatory_requirements": regulatory_requirements,
                },
                requirement_group_create_params.RequirementGroupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RequirementGroup,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementGroup:
        """
        Get a single requirement group by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/requirement_groups/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RequirementGroup,
        )

    def update(
        self,
        id: str,
        *,
        customer_reference: str | Omit = omit,
        regulatory_requirements: Iterable[requirement_group_update_params.RegulatoryRequirement] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementGroup:
        """
        Update requirement values in requirement group

        Args:
          customer_reference: Reference for the customer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/requirement_groups/{id}",
            body=maybe_transform(
                {
                    "customer_reference": customer_reference,
                    "regulatory_requirements": regulatory_requirements,
                },
                requirement_group_update_params.RequirementGroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RequirementGroup,
        )

    def list(
        self,
        *,
        filter: requirement_group_list_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementGroupListResponse:
        """
        List requirement groups

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[country_code], filter[phone_number_type], filter[action], filter[status],
              filter[customer_reference]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/requirement_groups",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"filter": filter}, requirement_group_list_params.RequirementGroupListParams),
            ),
            cast_to=RequirementGroupListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementGroup:
        """
        Delete a requirement group by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/requirement_groups/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RequirementGroup,
        )

    def submit_for_approval(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementGroup:
        """
        Submit a Requirement Group for Approval

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/requirement_groups/{id}/submit_for_approval",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RequirementGroup,
        )


class AsyncRequirementGroupsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRequirementGroupsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRequirementGroupsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRequirementGroupsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRequirementGroupsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        action: Literal["ordering", "porting"],
        country_code: str,
        phone_number_type: Literal["local", "toll_free", "mobile", "national", "shared_cost"],
        customer_reference: str | Omit = omit,
        regulatory_requirements: Iterable[requirement_group_create_params.RegulatoryRequirement] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementGroup:
        """
        Create a new requirement group

        Args:
          country_code: ISO alpha 2 country code

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/requirement_groups",
            body=await async_maybe_transform(
                {
                    "action": action,
                    "country_code": country_code,
                    "phone_number_type": phone_number_type,
                    "customer_reference": customer_reference,
                    "regulatory_requirements": regulatory_requirements,
                },
                requirement_group_create_params.RequirementGroupCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RequirementGroup,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementGroup:
        """
        Get a single requirement group by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/requirement_groups/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RequirementGroup,
        )

    async def update(
        self,
        id: str,
        *,
        customer_reference: str | Omit = omit,
        regulatory_requirements: Iterable[requirement_group_update_params.RegulatoryRequirement] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementGroup:
        """
        Update requirement values in requirement group

        Args:
          customer_reference: Reference for the customer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/requirement_groups/{id}",
            body=await async_maybe_transform(
                {
                    "customer_reference": customer_reference,
                    "regulatory_requirements": regulatory_requirements,
                },
                requirement_group_update_params.RequirementGroupUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RequirementGroup,
        )

    async def list(
        self,
        *,
        filter: requirement_group_list_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementGroupListResponse:
        """
        List requirement groups

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[country_code], filter[phone_number_type], filter[action], filter[status],
              filter[customer_reference]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/requirement_groups",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"filter": filter}, requirement_group_list_params.RequirementGroupListParams
                ),
            ),
            cast_to=RequirementGroupListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementGroup:
        """
        Delete a requirement group by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/requirement_groups/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RequirementGroup,
        )

    async def submit_for_approval(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequirementGroup:
        """
        Submit a Requirement Group for Approval

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/requirement_groups/{id}/submit_for_approval",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RequirementGroup,
        )


class RequirementGroupsResourceWithRawResponse:
    def __init__(self, requirement_groups: RequirementGroupsResource) -> None:
        self._requirement_groups = requirement_groups

        self.create = to_raw_response_wrapper(
            requirement_groups.create,
        )
        self.retrieve = to_raw_response_wrapper(
            requirement_groups.retrieve,
        )
        self.update = to_raw_response_wrapper(
            requirement_groups.update,
        )
        self.list = to_raw_response_wrapper(
            requirement_groups.list,
        )
        self.delete = to_raw_response_wrapper(
            requirement_groups.delete,
        )
        self.submit_for_approval = to_raw_response_wrapper(
            requirement_groups.submit_for_approval,
        )


class AsyncRequirementGroupsResourceWithRawResponse:
    def __init__(self, requirement_groups: AsyncRequirementGroupsResource) -> None:
        self._requirement_groups = requirement_groups

        self.create = async_to_raw_response_wrapper(
            requirement_groups.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            requirement_groups.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            requirement_groups.update,
        )
        self.list = async_to_raw_response_wrapper(
            requirement_groups.list,
        )
        self.delete = async_to_raw_response_wrapper(
            requirement_groups.delete,
        )
        self.submit_for_approval = async_to_raw_response_wrapper(
            requirement_groups.submit_for_approval,
        )


class RequirementGroupsResourceWithStreamingResponse:
    def __init__(self, requirement_groups: RequirementGroupsResource) -> None:
        self._requirement_groups = requirement_groups

        self.create = to_streamed_response_wrapper(
            requirement_groups.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            requirement_groups.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            requirement_groups.update,
        )
        self.list = to_streamed_response_wrapper(
            requirement_groups.list,
        )
        self.delete = to_streamed_response_wrapper(
            requirement_groups.delete,
        )
        self.submit_for_approval = to_streamed_response_wrapper(
            requirement_groups.submit_for_approval,
        )


class AsyncRequirementGroupsResourceWithStreamingResponse:
    def __init__(self, requirement_groups: AsyncRequirementGroupsResource) -> None:
        self._requirement_groups = requirement_groups

        self.create = async_to_streamed_response_wrapper(
            requirement_groups.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            requirement_groups.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            requirement_groups.update,
        )
        self.list = async_to_streamed_response_wrapper(
            requirement_groups.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            requirement_groups.delete,
        )
        self.submit_for_approval = async_to_streamed_response_wrapper(
            requirement_groups.submit_for_approval,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr
from .number_pool_settings_param import NumberPoolSettingsParam
from .url_shortener_settings_param import URLShortenerSettingsParam

__all__ = ["MessagingProfileCreateParams"]


class MessagingProfileCreateParams(TypedDict, total=False):
    name: Required[str]
    """A user friendly name for the messaging profile."""

    whitelisted_destinations: Required[SequenceNotStr[str]]
    """Destinations to which the messaging profile is allowed to send.

    The elements in the list must be valid ISO 3166-1 alpha-2 country codes. If set
    to `["*"]` all destinations will be allowed.
    """

    alpha_sender: Optional[str]
    """
    The alphanumeric sender ID to use when sending to destinations that require an
    alphanumeric sender ID.
    """

    daily_spend_limit: str
    """
    The maximum amount of money (in USD) that can be spent by this profile before
    midnight UTC.
    """

    daily_spend_limit_enabled: bool
    """Whether to enforce the value configured by `daily_spend_limit`."""

    enabled: bool
    """Specifies whether the messaging profile is enabled or not."""

    mms_fall_back_to_sms: bool
    """enables SMS fallback for MMS messages."""

    mms_transcoding: bool
    """enables automated resizing of MMS media."""

    mobile_only: bool
    """Send messages only to mobile phone numbers."""

    number_pool_settings: Optional[NumberPoolSettingsParam]
    """
    Number Pool allows you to send messages from a pool of numbers of different
    types, assigning weights to each type. The pool consists of all the long code
    and toll free numbers assigned to the messaging profile.

    To disable this feature, set the object field to `null`.
    """

    smart_encoding: bool
    """Enables automatic character encoding optimization for SMS messages.

    When enabled, the system automatically selects the most efficient encoding
    (GSM-7 or UCS-2) based on message content to maximize character limits and
    minimize costs.
    """

    url_shortener_settings: Optional[URLShortenerSettingsParam]
    """
    The URL shortener feature allows automatic replacement of URLs that were
    generated using a public URL shortener service. Some examples include bit.do,
    bit.ly, goo.gl, ht.ly, is.gd, ow.ly, rebrand.ly, t.co, tiny.cc, and tinyurl.com.
    Such URLs are replaced with with links generated by Telnyx. The use of custom
    links can improve branding and message deliverability.

    To disable this feature, set the object field to `null`.
    """

    webhook_api_version: Literal["1", "2", "2010-04-01"]
    """
    Determines which webhook format will be used, Telnyx API v1, v2, or a legacy
    2010-04-01 format.
    """

    webhook_failover_url: Optional[str]
    """
    The failover URL where webhooks related to this messaging profile will be sent
    if sending to the primary URL fails.
    """

    webhook_url: Optional[str]
    """The URL where webhooks related to this messaging profile will be sent."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["URLShortenerSettingsParam"]


class URLShortenerSettingsParam(TypedDict, total=False):
    """
    The URL shortener feature allows automatic replacement of URLs that were generated using
    a public URL shortener service. Some examples include bit.do, bit.ly, goo.gl, ht.ly,
    is.gd, ow.ly, rebrand.ly, t.co, tiny.cc, and tinyurl.com. Such URLs are replaced with
    with links generated by Telnyx. The use of custom links can improve branding and message
    deliverability.

    To disable this feature, set the object field to `null`.
    """

    domain: Required[str]
    """One of the domains provided by the Telnyx URL shortener service."""

    prefix: str
    """
    Optional prefix that can be used to identify your brand, and will appear in the
    Telnyx generated URLs after the domain name.
    """

    replace_blacklist_only: bool
    """
    Use the link replacement tool only for links that are specifically blacklisted
    by Telnyx.
    """

    send_webhooks: bool
    """Receive webhooks for when your replaced links are clicked.

    Webhooks are sent to the webhooks on the messaging profile.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .reports import (
    ReportsResource,
    AsyncReportsResource,
    ReportsResourceWithRawResponse,
    AsyncReportsResourceWithRawResponse,
    ReportsResourceWithStreamingResponse,
    AsyncReportsResourceWithStreamingResponse,
)
from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .loa_configurations import (
    LoaConfigurationsResource,
    AsyncLoaConfigurationsResource,
    LoaConfigurationsResourceWithRawResponse,
    AsyncLoaConfigurationsResourceWithRawResponse,
    LoaConfigurationsResourceWithStreamingResponse,
    AsyncLoaConfigurationsResourceWithStreamingResponse,
)
from ...types.porting_list_uk_carriers_response import PortingListUkCarriersResponse

__all__ = ["PortingResource", "AsyncPortingResource"]


class PortingResource(SyncAPIResource):
    @cached_property
    def events(self) -> EventsResource:
        return EventsResource(self._client)

    @cached_property
    def reports(self) -> ReportsResource:
        return ReportsResource(self._client)

    @cached_property
    def loa_configurations(self) -> LoaConfigurationsResource:
        return LoaConfigurationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> PortingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PortingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PortingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PortingResourceWithStreamingResponse(self)

    def list_uk_carriers(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortingListUkCarriersResponse:
        """List available carriers in the UK."""
        return self._get(
            "/porting/uk_carriers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PortingListUkCarriersResponse,
        )


class AsyncPortingResource(AsyncAPIResource):
    @cached_property
    def events(self) -> AsyncEventsResource:
        return AsyncEventsResource(self._client)

    @cached_property
    def reports(self) -> AsyncReportsResource:
        return AsyncReportsResource(self._client)

    @cached_property
    def loa_configurations(self) -> AsyncLoaConfigurationsResource:
        return AsyncLoaConfigurationsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPortingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPortingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPortingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPortingResourceWithStreamingResponse(self)

    async def list_uk_carriers(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortingListUkCarriersResponse:
        """List available carriers in the UK."""
        return await self._get(
            "/porting/uk_carriers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PortingListUkCarriersResponse,
        )


class PortingResourceWithRawResponse:
    def __init__(self, porting: PortingResource) -> None:
        self._porting = porting

        self.list_uk_carriers = to_raw_response_wrapper(
            porting.list_uk_carriers,
        )

    @cached_property
    def events(self) -> EventsResourceWithRawResponse:
        return EventsResourceWithRawResponse(self._porting.events)

    @cached_property
    def reports(self) -> ReportsResourceWithRawResponse:
        return ReportsResourceWithRawResponse(self._porting.reports)

    @cached_property
    def loa_configurations(self) -> LoaConfigurationsResourceWithRawResponse:
        return LoaConfigurationsResourceWithRawResponse(self._porting.loa_configurations)


class AsyncPortingResourceWithRawResponse:
    def __init__(self, porting: AsyncPortingResource) -> None:
        self._porting = porting

        self.list_uk_carriers = async_to_raw_response_wrapper(
            porting.list_uk_carriers,
        )

    @cached_property
    def events(self) -> AsyncEventsResourceWithRawResponse:
        return AsyncEventsResourceWithRawResponse(self._porting.events)

    @cached_property
    def reports(self) -> AsyncReportsResourceWithRawResponse:
        return AsyncReportsResourceWithRawResponse(self._porting.reports)

    @cached_property
    def loa_configurations(self) -> AsyncLoaConfigurationsResourceWithRawResponse:
        return AsyncLoaConfigurationsResourceWithRawResponse(self._porting.loa_configurations)


class PortingResourceWithStreamingResponse:
    def __init__(self, porting: PortingResource) -> None:
        self._porting = porting

        self.list_uk_carriers = to_streamed_response_wrapper(
            porting.list_uk_carriers,
        )

    @cached_property
    def events(self) -> EventsResourceWithStreamingResponse:
        return EventsResourceWithStreamingResponse(self._porting.events)

    @cached_property
    def reports(self) -> ReportsResourceWithStreamingResponse:
        return ReportsResourceWithStreamingResponse(self._porting.reports)

    @cached_property
    def loa_configurations(self) -> LoaConfigurationsResourceWithStreamingResponse:
        return LoaConfigurationsResourceWithStreamingResponse(self._porting.loa_configurations)


class AsyncPortingResourceWithStreamingResponse:
    def __init__(self, porting: AsyncPortingResource) -> None:
        self._porting = porting

        self.list_uk_carriers = async_to_streamed_response_wrapper(
            porting.list_uk_carriers,
        )

    @cached_property
    def events(self) -> AsyncEventsResourceWithStreamingResponse:
        return AsyncEventsResourceWithStreamingResponse(self._porting.events)

    @cached_property
    def reports(self) -> AsyncReportsResourceWithStreamingResponse:
        return AsyncReportsResourceWithStreamingResponse(self._porting.reports)

    @cached_property
    def loa_configurations(self) -> AsyncLoaConfigurationsResourceWithStreamingResponse:
        return AsyncLoaConfigurationsResourceWithStreamingResponse(self._porting.loa_configurations)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["BucketRetrieveResponse", "Data"]


class Data(BaseModel):
    created_at: datetime

    filename: str

    status: str

    error_reason: Optional[str] = None

    last_embedded_at: Optional[datetime] = None

    updated_at: Optional[datetime] = None


class BucketRetrieveResponse(BaseModel):
    data: List[Data]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ActionSendSipInfoParams"]


class ActionSendSipInfoParams(TypedDict, total=False):
    body: Required[str]
    """Content of the SIP INFO"""

    content_type: Required[str]
    """Content type of the INFO body.

    Must be MIME type compliant. There is a 1,400 bytes limit
    """

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["ParticipantParticipantsParams", "CustomHeader"]


class ParticipantParticipantsParams(TypedDict, total=False):
    account_sid: Required[str]

    amd_status_callback: Annotated[str, PropertyInfo(alias="AmdStatusCallback")]
    """The URL the result of answering machine detection will be sent to."""

    amd_status_callback_method: Annotated[Literal["GET", "POST"], PropertyInfo(alias="AmdStatusCallbackMethod")]
    """HTTP request type used for `AmdStatusCallback`. Defaults to `POST`."""

    beep: Annotated[Literal["true", "false", "onEnter", "onExit"], PropertyInfo(alias="Beep")]
    """
    Whether to play a notification beep to the conference when the participant
    enters and exits.
    """

    caller_id: Annotated[str, PropertyInfo(alias="CallerId")]
    """
    To be used as the caller id name (SIP From Display Name) presented to the
    destination (`To` number). The string should have a maximum of 128 characters,
    containing only letters, numbers, spaces, and `-_~!.+` special characters. If
    ommited, the display name will be the same as the number in the `From` field.
    """

    call_sid_to_coach: Annotated[str, PropertyInfo(alias="CallSidToCoach")]
    """The SID of the participant who is being coached.

    The participant being coached is the only participant who can hear the
    participant who is coaching.
    """

    cancel_playback_on_detect_message_end: Annotated[bool, PropertyInfo(alias="CancelPlaybackOnDetectMessageEnd")]
    """Whether to cancel ongoing playback on `greeting ended` detection.

    Defaults to `true`.
    """

    cancel_playback_on_machine_detection: Annotated[bool, PropertyInfo(alias="CancelPlaybackOnMachineDetection")]
    """Whether to cancel ongoing playback on `machine` detection. Defaults to `true`."""

    coaching: Annotated[bool, PropertyInfo(alias="Coaching")]
    """Whether the participant is coaching another call.

    When `true`, `CallSidToCoach` has to be given.
    """

    conference_record: Annotated[
        Literal["true", "false", "record-from-start", "do-not-record"], PropertyInfo(alias="ConferenceRecord")
    ]
    """Whether to record the conference the participant is joining.

    Defualts to `do-not-record`. The boolean values `true` and `false` are
    synonymous with `record-from-start` and `do-not-record` respectively.
    """

    conference_recording_status_callback: Annotated[str, PropertyInfo(alias="ConferenceRecordingStatusCallback")]
    """The URL the conference recording callbacks will be sent to."""

    conference_recording_status_callback_event: Annotated[
        str, PropertyInfo(alias="ConferenceRecordingStatusCallbackEvent")
    ]
    """
    The changes to the conference recording's state that should generate a call to
    `RecoridngStatusCallback`. Can be: `in-progress`, `completed` and `absent`.
    Separate multiple values with a space. Defaults to `completed`. `failed` and
    `absent` are synonymous.
    """

    conference_recording_status_callback_method: Annotated[
        Literal["GET", "POST"], PropertyInfo(alias="ConferenceRecordingStatusCallbackMethod")
    ]
    """HTTP request type used for `ConferenceRecordingStatusCallback`.

    Defaults to `POST`.
    """

    conference_recording_timeout: Annotated[int, PropertyInfo(alias="ConferenceRecordingTimeout")]
    """
    The number of seconds that Telnyx will wait for the recording to be stopped if
    silence is detected. The timer only starts when the speech is detected. Please
    note that the transcription is used to detect silence and the related charge
    will be applied. The minimum value is 0. The default value is 0 (infinite)
    """

    conference_status_callback: Annotated[str, PropertyInfo(alias="ConferenceStatusCallback")]
    """The URL the conference callbacks will be sent to."""

    conference_status_callback_event: Annotated[str, PropertyInfo(alias="ConferenceStatusCallbackEvent")]
    """
    The changes to the conference's state that should generate a call to
    `ConferenceStatusCallback`. Can be: `start`, `end`, `join` and `leave`. Separate
    multiple values with a space. By default no callbacks are sent.
    """

    conference_status_callback_method: Annotated[
        Literal["GET", "POST"], PropertyInfo(alias="ConferenceStatusCallbackMethod")
    ]
    """HTTP request type used for `ConferenceStatusCallback`. Defaults to `POST`."""

    conference_trim: Annotated[Literal["trim-silence", "do-not-trim"], PropertyInfo(alias="ConferenceTrim")]
    """Whether to trim any leading and trailing silence from the conference recording.

    Defaults to `trim-silence`.
    """

    custom_headers: Annotated[Iterable[CustomHeader], PropertyInfo(alias="CustomHeaders")]
    """Custom HTTP headers to be sent with the call.

    Each header should be an object with 'name' and 'value' properties.
    """

    early_media: Annotated[bool, PropertyInfo(alias="EarlyMedia")]
    """
    Whether participant shall be bridged to conference before the participant
    answers (from early media if available). Defaults to `false`.
    """

    end_conference_on_exit: Annotated[bool, PropertyInfo(alias="EndConferenceOnExit")]
    """Whether to end the conference when the participant leaves. Defaults to `false`."""

    from_: Annotated[str, PropertyInfo(alias="From")]
    """The phone number of the party that initiated the call.

    Phone numbers are formatted with a `+` and country code.
    """

    label: Annotated[str, PropertyInfo(alias="Label")]
    """A unique label for the participant that will be added to the conference.

    The label can be used to reference the participant for updates via the TeXML
    REST API.
    """

    machine_detection: Annotated[Literal["Enable", "DetectMessageEnd"], PropertyInfo(alias="MachineDetection")]
    """Whether to detect if a human or an answering machine picked up the call.

    Use `Enable` if you would like to ne notified as soon as the called party is
    identified. Use `DetectMessageEnd`, if you would like to leave a message on an
    answering machine.
    """

    machine_detection_silence_timeout: Annotated[int, PropertyInfo(alias="MachineDetectionSilenceTimeout")]
    """If initial silence duration is greater than this value, consider it a machine.

    Ignored when `premium` detection is used.
    """

    machine_detection_speech_end_threshold: Annotated[int, PropertyInfo(alias="MachineDetectionSpeechEndThreshold")]
    """
    Silence duration threshold after a greeting message or voice for it be
    considered human. Ignored when `premium` detection is used.
    """

    machine_detection_speech_threshold: Annotated[int, PropertyInfo(alias="MachineDetectionSpeechThreshold")]
    """Maximum threshold of a human greeting.

    If greeting longer than this value, considered machine. Ignored when `premium`
    detection is used.
    """

    machine_detection_timeout: Annotated[int, PropertyInfo(alias="MachineDetectionTimeout")]
    """
    How long answering machine detection should go on for before sending an
    `Unknown` result. Given in milliseconds.
    """

    max_participants: Annotated[int, PropertyInfo(alias="MaxParticipants")]
    """The maximum number of participants in the conference.

    Can be a positive integer from 2 to 800. The default value is 250.
    """

    muted: Annotated[bool, PropertyInfo(alias="Muted")]
    """Whether the participant should be muted."""

    preferred_codecs: Annotated[str, PropertyInfo(alias="PreferredCodecs")]
    """The list of comma-separated codecs to be offered on a call."""

    record: Annotated[bool, PropertyInfo(alias="Record")]
    """Whether to record the entire participant's call leg. Defaults to `false`."""

    recording_channels: Annotated[Literal["mono", "dual"], PropertyInfo(alias="RecordingChannels")]
    """The number of channels in the final recording. Defaults to `mono`."""

    recording_status_callback: Annotated[str, PropertyInfo(alias="RecordingStatusCallback")]
    """The URL the recording callbacks will be sent to."""

    recording_status_callback_event: Annotated[str, PropertyInfo(alias="RecordingStatusCallbackEvent")]
    """
    The changes to the recording's state that should generate a call to
    `RecoridngStatusCallback`. Can be: `in-progress`, `completed` and `absent`.
    Separate multiple values with a space. Defaults to `completed`.
    """

    recording_status_callback_method: Annotated[
        Literal["GET", "POST"], PropertyInfo(alias="RecordingStatusCallbackMethod")
    ]
    """HTTP request type used for `RecordingStatusCallback`. Defaults to `POST`."""

    recording_track: Annotated[Literal["inbound", "outbound", "both"], PropertyInfo(alias="RecordingTrack")]
    """The audio track to record for the call. The default is `both`."""

    sip_auth_password: Annotated[str, PropertyInfo(alias="SipAuthPassword")]
    """The password to use for SIP authentication."""

    sip_auth_username: Annotated[str, PropertyInfo(alias="SipAuthUsername")]
    """The username to use for SIP authentication."""

    start_conference_on_enter: Annotated[bool, PropertyInfo(alias="StartConferenceOnEnter")]
    """Whether to start the conference when the participant enters.

    Defaults to `true`.
    """

    status_callback: Annotated[str, PropertyInfo(alias="StatusCallback")]
    """URL destination for Telnyx to send status callback events to for the call."""

    status_callback_event: Annotated[str, PropertyInfo(alias="StatusCallbackEvent")]
    """The changes to the call's state that should generate a call to `StatusCallback`.

    Can be: `initiated`, `ringing`, `answered`, and `completed`. Separate multiple
    values with a space. The default value is `completed`.
    """

    status_callback_method: Annotated[Literal["GET", "POST"], PropertyInfo(alias="StatusCallbackMethod")]
    """HTTP request type used for `StatusCallback`."""

    time_limit: Annotated[int, PropertyInfo(alias="TimeLimit")]
    """The maximum duration of the call in seconds."""

    timeout_seconds: Annotated[int, PropertyInfo(alias="Timeout")]
    """
    The number of seconds that we should allow the phone to ring before assuming
    there is no answer. Can be an integer between 5 and 120, inclusive. The default
    value is 30.
    """

    to: Annotated[str, PropertyInfo(alias="To")]
    """The phone number of the called party.

    Phone numbers are formatted with a `+` and country code.
    """

    trim: Annotated[Literal["trim-silence", "do-not-trim"], PropertyInfo(alias="Trim")]
    """Whether to trim any leading and trailing silence from the recording.

    Defaults to `trim-silence`.
    """

    wait_url: Annotated[str, PropertyInfo(alias="WaitUrl")]
    """
    The URL to call for an audio file to play while the participant is waiting for
    the conference to start.
    """


class CustomHeader(TypedDict, total=False):
    name: Required[str]
    """The name of the custom header"""

    value: Required[str]
    """The value of the custom header"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["CloudflareSyncStatus"]

CloudflareSyncStatus: TypeAlias = Literal["pending", "added"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import numbers_feature_create_params
from .._types import Body, Query, Headers, NotGiven, SequenceNotStr, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.numbers_feature_create_response import NumbersFeatureCreateResponse

__all__ = ["NumbersFeaturesResource", "AsyncNumbersFeaturesResource"]


class NumbersFeaturesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> NumbersFeaturesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return NumbersFeaturesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NumbersFeaturesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return NumbersFeaturesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        phone_numbers: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumbersFeatureCreateResponse:
        """
        Retrieve the features for a list of numbers

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/numbers_features",
            body=maybe_transform(
                {"phone_numbers": phone_numbers}, numbers_feature_create_params.NumbersFeatureCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumbersFeatureCreateResponse,
        )


class AsyncNumbersFeaturesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncNumbersFeaturesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNumbersFeaturesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNumbersFeaturesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncNumbersFeaturesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        phone_numbers: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumbersFeatureCreateResponse:
        """
        Retrieve the features for a list of numbers

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/numbers_features",
            body=await async_maybe_transform(
                {"phone_numbers": phone_numbers}, numbers_feature_create_params.NumbersFeatureCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumbersFeatureCreateResponse,
        )


class NumbersFeaturesResourceWithRawResponse:
    def __init__(self, numbers_features: NumbersFeaturesResource) -> None:
        self._numbers_features = numbers_features

        self.create = to_raw_response_wrapper(
            numbers_features.create,
        )


class AsyncNumbersFeaturesResourceWithRawResponse:
    def __init__(self, numbers_features: AsyncNumbersFeaturesResource) -> None:
        self._numbers_features = numbers_features

        self.create = async_to_raw_response_wrapper(
            numbers_features.create,
        )


class NumbersFeaturesResourceWithStreamingResponse:
    def __init__(self, numbers_features: NumbersFeaturesResource) -> None:
        self._numbers_features = numbers_features

        self.create = to_streamed_response_wrapper(
            numbers_features.create,
        )


class AsyncNumbersFeaturesResourceWithStreamingResponse:
    def __init__(self, numbers_features: AsyncNumbersFeaturesResource) -> None:
        self._numbers_features = numbers_features

        self.create = async_to_streamed_response_wrapper(
            numbers_features.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["AccountRetrieveTranscriptionsJsonResponse", "Transcription"]


class Transcription(BaseModel):
    account_sid: Optional[str] = None

    api_version: Optional[str] = None
    """The version of the API that was used to make the request."""

    call_sid: Optional[str] = None

    date_created: Optional[datetime] = None

    date_updated: Optional[datetime] = None

    duration: Optional[str] = None
    """The duration of this recording, given in seconds."""

    recording_sid: Optional[str] = None
    """Identifier of a resource."""

    sid: Optional[str] = None
    """Identifier of a resource."""

    status: Optional[Literal["in-progress", "completed"]] = None
    """The status of the recording transcriptions.

    The transcription text will be available only when the status is completed.
    """

    transcription_text: Optional[str] = None
    """The recording's transcribed text"""

    uri: Optional[str] = None
    """The relative URI for the recording transcription resource."""


class AccountRetrieveTranscriptionsJsonResponse(BaseModel):
    end: Optional[int] = None
    """The number of the last element on the page, zero-indexed"""

    first_page_uri: Optional[str] = None
    """Relative uri to the first page of the query results"""

    next_page_uri: Optional[str] = None
    """Relative uri to the next page of the query results"""

    page: Optional[int] = None
    """Current page number, zero-indexed."""

    page_size: Optional[int] = None
    """The number of items on the page"""

    previous_page_uri: Optional[str] = None
    """Relative uri to the previous page of the query results"""

    start: Optional[int] = None
    """The number of the first element on the page, zero-indexed."""

    transcriptions: Optional[List[Transcription]] = None

    uri: Optional[str] = None
    """The URI of the current page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .portout_details import PortoutDetails

__all__ = ["PortoutRetrieveResponse"]


class PortoutRetrieveResponse(BaseModel):
    data: Optional[PortoutDetails] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .porting_order_misc import PortingOrderMisc
from .porting_order_end_user import PortingOrderEndUser
from .porting_order_documents import PortingOrderDocuments
from .porting_order_messaging import PortingOrderMessaging
from .porting_order_requirement import PortingOrderRequirement
from .porting_order_user_feedback import PortingOrderUserFeedback
from .shared.porting_order_status import PortingOrderStatus
from .porting_order_activation_settings import PortingOrderActivationSettings
from .porting_order_phone_number_configuration import PortingOrderPhoneNumberConfiguration

__all__ = ["PortingOrder"]


class PortingOrder(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies this porting order"""

    activation_settings: Optional[PortingOrderActivationSettings] = None

    additional_steps: Optional[List[Literal["associated_phone_numbers", "phone_number_verification_codes"]]] = None
    """
    For specific porting orders, we may require additional steps to be taken before
    submitting the porting order.
    """

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    customer_group_reference: Optional[str] = None
    """A customer-specified group reference for customer bookkeeping purposes"""

    customer_reference: Optional[str] = None
    """A customer-specified reference number for customer bookkeeping purposes"""

    description: Optional[str] = None
    """A description of the porting order"""

    documents: Optional[PortingOrderDocuments] = None
    """Can be specified directly or via the `requirement_group_id` parameter."""

    end_user: Optional[PortingOrderEndUser] = None

    messaging: Optional[PortingOrderMessaging] = None
    """Information about messaging porting process."""

    misc: Optional[PortingOrderMisc] = None

    old_service_provider_ocn: Optional[str] = None
    """Identifies the old service provider"""

    parent_support_key: Optional[str] = None
    """
    A key to reference for the porting order group when contacting Telnyx customer
    support. This information is not available for porting orders in `draft` state
    """

    phone_number_configuration: Optional[PortingOrderPhoneNumberConfiguration] = None

    phone_number_type: Optional[Literal["landline", "local", "mobile", "national", "shared_cost", "toll_free"]] = None
    """The type of the phone number"""

    porting_phone_numbers_count: Optional[int] = None
    """Count of phone numbers associated with this porting order"""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    requirements: Optional[List[PortingOrderRequirement]] = None
    """List of documentation requirements for porting numbers.

    Can be set directly or via the `requirement_group_id` parameter.
    """

    requirements_met: Optional[bool] = None
    """Is true when the required documentation is met"""

    status: Optional[PortingOrderStatus] = None
    """Porting order status"""

    support_key: Optional[str] = None
    """A key to reference this porting order when contacting Telnyx customer support.

    This information is not available in draft porting orders.
    """

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    user_feedback: Optional[PortingOrderUserFeedback] = None

    user_id: Optional[str] = None
    """Identifies the user (or organization) who requested the porting order"""

    webhook_url: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import channel_zone_list_params, channel_zone_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.channel_zone_list_response import ChannelZoneListResponse
from ..types.channel_zone_update_response import ChannelZoneUpdateResponse

__all__ = ["ChannelZonesResource", "AsyncChannelZonesResource"]


class ChannelZonesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChannelZonesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ChannelZonesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChannelZonesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ChannelZonesResourceWithStreamingResponse(self)

    def update(
        self,
        channel_zone_id: str,
        *,
        channels: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChannelZoneUpdateResponse:
        """Update the number of Voice Channels for the Non-US Zones.

        This allows your
        account to handle multiple simultaneous inbound calls to Non-US numbers. Use
        this endpoint to increase or decrease your capacity based on expected call
        volume.

        Args:
          channels: The number of reserved channels

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not channel_zone_id:
            raise ValueError(f"Expected a non-empty value for `channel_zone_id` but received {channel_zone_id!r}")
        return self._put(
            f"/channel_zones/{channel_zone_id}",
            body=maybe_transform({"channels": channels}, channel_zone_update_params.ChannelZoneUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChannelZoneUpdateResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[ChannelZoneListResponse]:
        """Returns the non-US voice channels for your account.

        voice channels allow you to
        use Channel Billing for calls to your Telnyx phone numbers. Please check the
        <a href="https://support.telnyx.com/en/articles/8428806-global-channel-billing">Telnyx
        Support Articles</a> section for full information and examples of how to utilize
        Channel Billing.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/channel_zones",
            page=SyncDefaultFlatPagination[ChannelZoneListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    channel_zone_list_params.ChannelZoneListParams,
                ),
            ),
            model=ChannelZoneListResponse,
        )


class AsyncChannelZonesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChannelZonesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChannelZonesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChannelZonesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncChannelZonesResourceWithStreamingResponse(self)

    async def update(
        self,
        channel_zone_id: str,
        *,
        channels: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChannelZoneUpdateResponse:
        """Update the number of Voice Channels for the Non-US Zones.

        This allows your
        account to handle multiple simultaneous inbound calls to Non-US numbers. Use
        this endpoint to increase or decrease your capacity based on expected call
        volume.

        Args:
          channels: The number of reserved channels

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not channel_zone_id:
            raise ValueError(f"Expected a non-empty value for `channel_zone_id` but received {channel_zone_id!r}")
        return await self._put(
            f"/channel_zones/{channel_zone_id}",
            body=await async_maybe_transform(
                {"channels": channels}, channel_zone_update_params.ChannelZoneUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChannelZoneUpdateResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ChannelZoneListResponse, AsyncDefaultFlatPagination[ChannelZoneListResponse]]:
        """Returns the non-US voice channels for your account.

        voice channels allow you to
        use Channel Billing for calls to your Telnyx phone numbers. Please check the
        <a href="https://support.telnyx.com/en/articles/8428806-global-channel-billing">Telnyx
        Support Articles</a> section for full information and examples of how to utilize
        Channel Billing.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/channel_zones",
            page=AsyncDefaultFlatPagination[ChannelZoneListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    channel_zone_list_params.ChannelZoneListParams,
                ),
            ),
            model=ChannelZoneListResponse,
        )


class ChannelZonesResourceWithRawResponse:
    def __init__(self, channel_zones: ChannelZonesResource) -> None:
        self._channel_zones = channel_zones

        self.update = to_raw_response_wrapper(
            channel_zones.update,
        )
        self.list = to_raw_response_wrapper(
            channel_zones.list,
        )


class AsyncChannelZonesResourceWithRawResponse:
    def __init__(self, channel_zones: AsyncChannelZonesResource) -> None:
        self._channel_zones = channel_zones

        self.update = async_to_raw_response_wrapper(
            channel_zones.update,
        )
        self.list = async_to_raw_response_wrapper(
            channel_zones.list,
        )


class ChannelZonesResourceWithStreamingResponse:
    def __init__(self, channel_zones: ChannelZonesResource) -> None:
        self._channel_zones = channel_zones

        self.update = to_streamed_response_wrapper(
            channel_zones.update,
        )
        self.list = to_streamed_response_wrapper(
            channel_zones.list,
        )


class AsyncChannelZonesResourceWithStreamingResponse:
    def __init__(self, channel_zones: AsyncChannelZonesResource) -> None:
        self._channel_zones = channel_zones

        self.update = async_to_streamed_response_wrapper(
            channel_zones.update,
        )
        self.list = async_to_streamed_response_wrapper(
            channel_zones.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .messaging import (
    MessagingResource,
    AsyncMessagingResource,
    MessagingResourceWithRawResponse,
    AsyncMessagingResourceWithRawResponse,
    MessagingResourceWithStreamingResponse,
    AsyncMessagingResourceWithStreamingResponse,
)
from .mobile_phone_numbers import (
    MobilePhoneNumbersResource,
    AsyncMobilePhoneNumbersResource,
    MobilePhoneNumbersResourceWithRawResponse,
    AsyncMobilePhoneNumbersResourceWithRawResponse,
    MobilePhoneNumbersResourceWithStreamingResponse,
    AsyncMobilePhoneNumbersResourceWithStreamingResponse,
)

__all__ = [
    "MessagingResource",
    "AsyncMessagingResource",
    "MessagingResourceWithRawResponse",
    "AsyncMessagingResourceWithRawResponse",
    "MessagingResourceWithStreamingResponse",
    "AsyncMessagingResourceWithStreamingResponse",
    "MobilePhoneNumbersResource",
    "AsyncMobilePhoneNumbersResource",
    "MobilePhoneNumbersResourceWithRawResponse",
    "AsyncMobilePhoneNumbersResourceWithRawResponse",
    "MobilePhoneNumbersResourceWithStreamingResponse",
    "AsyncMobilePhoneNumbersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["MessagingNumbersBulkUpdateCreateParams"]


class MessagingNumbersBulkUpdateCreateParams(TypedDict, total=False):
    messaging_profile_id: Required[str]
    """Configure the messaging profile these phone numbers are assigned to:

    - Set this field to `""` to unassign each number from their respective messaging
      profile
    - Set this field to a quoted UUID of a messaging profile to assign these numbers
      to that messaging profile
    """

    numbers: Required[SequenceNotStr[str]]
    """The list of phone numbers to update."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .calls import (
    CallsResource,
    AsyncCallsResource,
    CallsResourceWithRawResponse,
    AsyncCallsResourceWithRawResponse,
    CallsResourceWithStreamingResponse,
    AsyncCallsResourceWithStreamingResponse,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "CallsResource",
    "AsyncCallsResource",
    "CallsResourceWithRawResponse",
    "AsyncCallsResourceWithRawResponse",
    "CallsResourceWithStreamingResponse",
    "AsyncCallsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_enqueued import CallEnqueued

__all__ = ["CallEnqueuedWebhookEvent"]


class CallEnqueuedWebhookEvent(BaseModel):
    data: Optional[CallEnqueued] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["DialogflowConnectionCreateResponse", "Data"]


class Data(BaseModel):
    connection_id: Optional[str] = None
    """Uniquely identifies a Telnyx application (Call Control)."""

    conversation_profile_id: Optional[str] = None
    """The id of a configured conversation profile on your Dialogflow account.

    (If you use Dialogflow CX, this param is required)
    """

    environment: Optional[str] = None
    """Which Dialogflow environment will be used."""

    record_type: Optional[str] = None

    service_account: Optional[str] = None
    """The JSON map to connect your Dialoglow account."""


class DialogflowConnectionCreateResponse(BaseModel):
    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel
from .mission_data import MissionData

__all__ = ["MissionRetrieveResponse"]


class MissionRetrieveResponse(BaseModel):
    data: MissionData



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["AuditEventListResponse", "Change"]


class Change(BaseModel):
    """Details of the changes made to a resource."""

    field: Optional[str] = None
    """The name of the field that was changed.

    May use the dot notation to indicate nested fields.
    """

    from_: Union[str, float, bool, Dict[str, object], List[Dict[str, object]], None] = FieldInfo(
        alias="from", default=None
    )
    """The previous value of the field. Can be any JSON type."""

    to: Union[str, float, bool, Dict[str, object], List[Dict[str, object]], None] = None
    """The new value of the field. Can be any JSON type."""


class AuditEventListResponse(BaseModel):
    id: Optional[str] = None
    """Unique identifier for the audit log entry."""

    alternate_resource_id: Optional[str] = None
    """
    An alternate identifier for a resource which may be considered unique enough to
    identify the resource but is not the primary identifier for the resource. For
    example, this field could be used to store the phone number value for a phone
    number when the primary database identifier is a separate distinct value.
    """

    change_made_by: Optional[Literal["telnyx", "account_manager", "account_owner", "organization_member"]] = None
    """
    Indicates if the change was made by Telnyx on your behalf, the organization
    owner, a member of your organization, or in the case of managed accounts, the
    account manager.
    """

    change_type: Optional[str] = None
    """The type of change that occurred."""

    changes: Optional[List[Change]] = None
    """Details of the changes made to the resource."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the change occurred."""

    organization_id: Optional[str] = None
    """Unique identifier for the organization that owns the resource."""

    record_type: Optional[str] = None
    """The type of the resource being audited."""

    resource_id: Optional[str] = None
    """Unique identifier for the resource that was changed."""

    user_id: Optional[str] = None
    """Unique identifier for the user who made the change."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ChargesBreakdownRetrieveParams"]


class ChargesBreakdownRetrieveParams(TypedDict, total=False):
    start_date: Required[Annotated[Union[str, date], PropertyInfo(format="iso8601")]]
    """Start date for the charges breakdown in ISO date format (YYYY-MM-DD)"""

    end_date: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """End date for the charges breakdown in ISO date format (YYYY-MM-DD).

    If not provided, defaults to start_date + 1 month. The date is exclusive, data
    for the end_date itself is not included in the report. The interval between
    start_date and end_date cannot exceed 31 days.
    """

    format: Literal["json", "csv"]
    """Response format"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ....._models import BaseModel
from .telco_data_usage_report_response import TelcoDataUsageReportResponse

__all__ = ["NumberLookupCreateResponse"]


class NumberLookupCreateResponse(BaseModel):
    data: Optional[TelcoDataUsageReportResponse] = None
    """Telco data usage report response"""
